/*
*  Non-Convex Constrained Variable-Size Simplex Algorithm Example
*  from F. J. Gould in "Numerical Methods for Non-linear Optimization"
*  (edited by F. A. Lootsma)
*
*  version 20.4.2002
*  (c) 2002 Ingo D. Rullhusen <d01c@uni-bremen.de>
* 
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
* 
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
* 
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
*/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <libvssa.h>


typedef struct
{
   double th1, th2;
} VECT;


const double xi[44] = {  8.0,  8.0, 10.0, 10.0,
                        10.0, 10.0, 12.0, 12.0,
                        12.0, 12.0, 14.0, 14.0,
                        14.0, 16.0, 16.0, 16.0,
                        18.0, 18.0, 20.0, 20.0,
                        20.0, 22.0, 22.0, 22.0,
                        24.0, 24.0, 24.0, 26.0,
                        26.0, 26.0, 28.0, 28.0,
                        30.0, 30.0, 30.0, 32.0,
                        32.0, 34.0, 36.0, 36.0,
                        38.0, 38.0, 40.0, 42.0  };

const double yi[44] = { 0.49, 0.49, 0.48, 0.47,
                        0.48, 0.47, 0.46, 0.46,
                        0.45, 0.43, 0.45, 0.43,
                        0.43, 0.44, 0.43, 0.43,
                        0.46, 0.45, 0.42, 0.42,
                        0.43, 0.41, 0.41, 0.40,
                        0.42, 0.40, 0.40, 0.41,
                        0.40, 0.41, 0.41, 0.40,
                        0.40, 0.40, 0.38, 0.41,
                        0.40, 0.40, 0.41, 0.38,
                        0.40, 0.40, 0.39, 0.39  };


double f( double *x, void *data )
{
   int i;
   double sum;
   VECT *v = (VECT*) x;
   (*((int*)data))++;

   sum = 0.0;
   for ( i=0; i<44; i++ ) sum +=pow( yi[i] - v->th1 - ( 0.49 - v->th1 ) * exp( -v->th2*(xi[i]-8.0) ), 2 );

   return(sum);
}


double g1( double *x, void *data )
{
   VECT *v = (VECT*) x;
   (*((int*)data))++;
   return( v->th1*v->th2 - 0.49*v->th2 + 0.09 );
}
 

double g2( double *x, void *data )
{
   VECT *v = (VECT*) x;
   (*((int*)data))++;
   return( -v->th1 + 0.40 );
}


int main()
{
   int major, minor, sub, i, fi, gi[4];
   VECT min, max, *v, *dv;
   MIN_PROBLEM_C mprob;
   MIN_SOLUTION_C *solution;

   LibVSSAversion(&major,&minor,&sub);
   printf("\nusing libvssa Version %i.%i.%i\n",major,minor,sub);

   mprob.n = 2;

   mprob.x_min = (double*) &min;
   mprob.x_max = (double*) &max;

   min.th1 = 0.41;
   min.th2 = 1.5;

   max.th1 = min.th1 + 0.1;
   max.th2 = min.th2 + 0.1;

   mprob.func_data = (void*)&fi;
   fi = 0;
   mprob.func = f;

   mprob.m = 2;
   mprob.ief = calloc( mprob.m, sizeof( double(*)( double*, void* ) ) );
   mprob.ief[0] = g1;
   mprob.ief[1] = g2;
   mprob.ief_data = calloc( mprob.m, sizeof(void*) );
   for ( i=0; i<mprob.m; i++ )
   {
      mprob.ief_data[i] = &gi[i];
      gi[i] = 0;
   }

   mprob.k = 0;
   mprob.eqf = NULL;

   mprob.maxitno_vssa = 5000;
   mprob.maxitno_penalty = 50;
   mprob.eps = 1e-14;
   
   mprob.alpha = 1.0;
   mprob.beta = 0.5;
   mprob.gamma = 2.0;

   mprob.r0 = 1e2;
   mprob.rf = 10.0;

   solution = CreateSolutionC( &mprob );

   printf("\nGould's non-convex example 4.4\n");
   printf(" no of function calls: %i\n",*(int*)mprob.func_data);
   printf(" termination code: %i no of iterations: %i\n",solution->term,solution->itnop);
   printf(" f=%f df=%f\n",solution->y,solution->dy);

   v  = (VECT*) solution->x;
   dv = (VECT*) solution->dx;
   printf(" th1=%f dth1=%f\n",v->th1,dv->th1);
   printf(" th2=%f dth2=%f\n",v->th2,dv->th2);

   printf("\n");
   printf(" data from %i inequation constraints\n",solution->m);
   for ( i=0; i<solution->m; i++ )
   {
      printf(" g%i=%f Lagrange=%f Lerror=%f\n",i+1,solution->dief[i],solution->ui[i],solution->uierr[i]);
   }
   printf("\n");

   DeleteSolutionC( &solution );

   return(0);
}
