/*
*  Constrained Variable-Size Simplex Algorithm Example
*  David M. Himmelblau in "Applied Nonlinear Programming"
*
*  version 20.4.2002
*  (c) 2002 Ingo D. Rullhusen <d01c@uni-bremen.de>
* 
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
* 
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
* 
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
*/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <libvssa.h>


typedef struct
{
   double x1, x2, x3;
} VECT;


double squ( double x ){ return(x*x); }


double f( double *x, void *data )
{
   VECT *v = (VECT*) x;
   (*((int*)data))++;
   return( 1000.0 - squ(v->x1) - 2.0*squ(v->x2) - squ(v->x3) - v->x1*v->x2
           - v->x1*v->x3 );
}


double h1( double *x, void *data )
{
   VECT *v = (VECT*) x;
   (*((int*)data))++;
   return( squ(v->x1) + squ(v->x2) + squ(v->x3) - 25.0 );
}


double h2( double *x, void *data )
{
   VECT *v = (VECT*) x;
   (*((int*)data))++;
   return( 8.0*v->x1 + 14.0*v->x2 + 7.0*v->x3 - 56.0 );
}


double g1( double *x, void *data )
{
   VECT *v = (VECT*) x;
   (*((int*)data))++;  
   return( - v->x1 );
}


double g2( double *x, void *data )
{
   VECT *v = (VECT*) x;
   (*((int*)data))++;  
   return( - v->x2 );
}


double g3( double *x, void *data )
{
   VECT *v = (VECT*) x;
   (*((int*)data))++;  
   return( - v->x3 );
}


int main()
{
   int major, minor, sub, i, fi, gi[3], hi[2];
   VECT min, max, *v, *dv;
   MIN_PROBLEM_C mprob;
   MIN_SOLUTION_C *solution;

   LibVSSAversion(&major,&minor,&sub);
   printf("\nusing libvssa Version %i.%i.%i\n",major,minor,sub);

   mprob.n = 3;

   mprob.x_min = (double*) &min;
   mprob.x_max = (double*) &max;

   min.x1 = 2.0;
   min.x2 = 2.0;
   min.x3 = 2.0;

   max.x1 = min.x1 + 0.1;
   max.x2 = min.x2 + 0.1;
   max.x3 = min.x3 + 0.1;

   mprob.func_data = (void*)&fi;
   fi = 0;
   mprob.func = f;

   mprob.m = 3;
   mprob.ief = calloc( mprob.m, sizeof( double(*)( double*, void* ) ) );
   mprob.ief[0] = g1;
   mprob.ief[1] = g2;
   mprob.ief[2] = g3;
   mprob.ief_data = calloc( mprob.m, sizeof(void*) );
   for ( i=0; i<mprob.m; i++ )
   {
      mprob.ief_data[i] = &gi[i];
      gi[i] = 0;
   }

   mprob.k = 2;
   mprob.eqf = calloc( mprob.k, sizeof( double(*)( double*, void* ) ) );
   mprob.eqf[0] = h1;
   mprob.eqf[1] = h2;
   mprob.eqf_data = calloc( mprob.k, sizeof(void*) );
   for ( i=0; i<mprob.k; i++ )
   {
      mprob.eqf_data[i] = &hi[i];
      hi[i] = 0;
   }

   mprob.maxitno_vssa = 5000;
   mprob.maxitno_penalty = 50;
   mprob.eps = 1e-10;
   
   mprob.alpha = 1.0;
   mprob.beta = 0.5;
   mprob.gamma = 2.0;

   mprob.r0 = 1e2;
   mprob.rf = 10.0;

   solution = CreateSolutionC( &mprob );

   printf("\nHimmelblau's problem 5/appendix a\n");
   printf(" no of function calls: %i\n",*(int*)mprob.func_data);
   printf(" termination code: %i no of iterations: %i\n",solution->term,solution->itnop);
   printf(" f=%f df=%f\n",solution->y,solution->dy);

   v  = (VECT*) solution->x;
   dv = (VECT*) solution->dx;
   printf(" x1=%f dx=%f\n",v->x1,dv->x1);
   printf(" x2=%f dx=%f\n",v->x2,dv->x2);
   printf(" x3=%f dx=%f\n",v->x3,dv->x3);

   printf("\n");
   printf(" data from %i inequation constraints\n",solution->m);
   for ( i=0; i<solution->m; i++ )
   {
      printf(" g%i=%f Lagrange=%f Lerror=%f\n",i+1,solution->dief[i],solution->ui[i],solution->uierr[i]);
   }

   printf("\n");
   printf(" data from %i equation constraints\n",solution->k);
   for ( i=0; i<solution->k; i++ )
   {
      printf(" h%i=%g Lagrange=%f Lerror=%f\n",i+1,solution->deqf[i],solution->ue[i],solution->ueerr[i]);
   }
   printf("\n");

   DeleteSolutionC( &solution );

   return(0);
}
