/*
*  Variable-Size Simplex Algorithm Examples from Nelder/Mead
*  version 28.4.2002
*  (c) 2002 Ingo D. Rullhusen <d01c@uni-bremen.de>
* 
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
* 
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
* 
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
*/


#include <stdio.h>
#include <stdlib.h>

#define __USE_BSD
#include <math.h>


#include <libvssa.h>


double Rosenbrock( double *x, void *data )
{
   ( *( (int*)data ) )++;
   return( 100.0*pow( x[1] - x[0]*x[0], 2 ) + pow( 1.0 - x[0], 2 ) );
}
/* Rosenbrock's parabolic valley */


double Powell( double *x, void *data )
{
   ( *( (int*)data ) )++;
   return( pow( x[0] + 10.0*x[1], 2 )
           + 5.0*pow( x[2] - x[3], 2 )
             + pow( x[1] - 2.0*x[2], 4 )
               + 10.0*pow( x[0] - x[3], 4 ) );
}
/* Powell's quartic function */


double Fletcher( double *x, void *data )
{
   ( *( (int*)data ) )++;
   return( 100.0*pow( x[2] - 10.0*( x[0]>0 ? atan( x[1]/x[0] )/(2.0*M_PI)
                                           : ( M_PI + atan( x[1]/x[0] ) )/(2.0*M_PI) ), 2 )
           + pow( sqrt( x[0]*x[0] + x[1]*x[1] ) - 1.0, 2 ) + x[2]*x[2]
         );
}
/* Fletcher and Powell's helical valley */


int main()
{
   int i;
   MIN_PROBLEM mprob;
   MIN_SOLUTION *solut;

   mprob.x_min = calloc( 4, sizeof(double) );
   mprob.x_max = calloc( 4, sizeof(double) );

   mprob.func_data = malloc( sizeof(int) );

   mprob.maxitno = 1000;
   mprob.eps = 1e-12;

   mprob.alpha = 1.0;
   mprob.beta = 0.5;
   mprob.gamma = 2.0;


/* Rosenbrock */
   mprob.n = 2;
   mprob.func = Rosenbrock;
   *(int*)mprob.func_data = 0;

   mprob.x_min[0] = -1.2;   mprob.x_max[0] = -0.7;
   mprob.x_min[1] =  1.0;   mprob.x_max[1] =  1.5;

   solut = CreateSolution(&mprob);

   printf("\nRosenbrock's parabolic valley\n");
   printf(" no of function calls: %i\n",*(int*)mprob.func_data);
   printf(" termination code: %i no of iterations: %i\n",solut->term,solut->itno);
   printf(" y=%f dy=%f\n",solut->y,solut->dy);
   for ( i=0; i<solut->n; i++ )
   {
      printf(" x[%i]=%f dx[%i]=%f\n",i,solut->x[i],i,solut->dx[i]);
   }

   DeleteSolution(&solut);


/* Powell */
   mprob.n = 4;
   mprob.func = Powell;
   *(int*)mprob.func_data = 0;

   mprob.x_min[0] =  3.0;   mprob.x_max[0] =  3.5;
   mprob.x_min[1] = -1.0;   mprob.x_max[1] = -0.5;
   mprob.x_min[2] =  0.0;   mprob.x_max[2] =  0.5;
   mprob.x_min[3] =  1.0;   mprob.x_max[3] =  1.5;

   solut = CreateSolution(&mprob);

   printf("\nPowell's quartic function\n");
   printf(" no of function calls: %i\n",*(int*)mprob.func_data);
   printf(" termination code: %i no of iterations: %i\n",solut->term,solut->itno);
   printf(" y=%f dy=%f\n",solut->y,solut->dy);
   for ( i=0; i<solut->n; i++ )
   {
      printf(" x[%i]=%f dx[%i]=%f\n",i,solut->x[i],i,solut->dx[i]);
   }

   DeleteSolution(&solut);


/* Fletcher and Powell's helical valley */
   mprob.n = 3;
   mprob.func = Fletcher;
   *(int*)mprob.func_data = 0;

   mprob.x_min[0] = -1.0;   mprob.x_max[0] = -0.5;
   mprob.x_min[1] =  0.0;   mprob.x_max[1] =  0.5;
   mprob.x_min[2] =  0.0;   mprob.x_max[2] =  0.5;

   solut = CreateSolution(&mprob);

   printf("\nFletcher and Powell's helical valley\n");
   printf(" no of function calls: %i\n",*(int*)mprob.func_data);
   printf(" termination code: %i no of iterations: %i\n",solut->term,solut->itno);
   printf(" y=%f dy=%f\n",solut->y,solut->dy);
   for ( i=0; i<solut->n; i++ )
   {
      printf(" x[%i]=%f dx[%i]=%f\n",i,solut->x[i],i,solut->dx[i]);
   }

   DeleteSolution(&solut);

   printf("\n");

   free( mprob.func_data );

   free( mprob.x_max );
   free( mprob.x_min );

   return(0);
}
