/*
*  Variable-Size Simplex Algorithm
*  version 27.10.2004
*  (c) 2002-2004 Ingo D. Rullhusen <d01c@uni-bremen.de>
* 
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
* 
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
* 
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
*/


#ifndef VSSA_H
#define VSSA_H

#ifdef __cplusplus
#define ExC_ extern "C"
#else
#define ExC_
#endif


ExC_ void LibVSSAversion( int *major, int *minor, int *sub );
/* return version */


/*
*   PART I: Base Algorithm for Unconstrained Minimization
*/


typedef struct
{
   int n;                                   /* no of variables           */
   double (*func)( double *x, void *data ); /* objective function        */
   void *func_data;                         /* objective function param  */
   double *x_min;                           /* x_min[0...n-1] min values */
   double *x_max;                           /* x_max[0...n-1] max values */
   int maxitno;                             /* max iteration no          */
   double eps;                              /* error of solution > 0     */
   double alpha;                            /* reflection coefficent > 0 */
   double beta;                             /* 0 < contraction coeff < 1 */
   double gamma;                            /* 1 < expansion coefficent  */
} MIN_PROBLEM;
/* define unconstrained minimization problem */


typedef struct
{
   int n;      /* no of variables                      */
   double **P; /* n+1 simplex points P[0...n][0...n-1] */
   double *y;  /* function values at n+1 points        */
   int yh;     /* max value position                   */
   int yl;     /* min value position                   */
   double *Ps, *Pss, *Pq, ys, yss; /* temps            */
} SIMPLEX;
/* define simplex */


ExC_ SIMPLEX* MakeFirstSimplex( MIN_PROBLEM *minprob );
/* generate first simplex for iteration */


ExC_ void DeleteSimplex( SIMPLEX **simp );
/* delete simplex */


ExC_ void VSSAstep( MIN_PROBLEM *minprob, SIMPLEX *simp );
/* one step of the variable-size simplex algorithm */


ExC_ int VSSAend( double epsilon, SIMPLEX *simp );
/* termination criterion, 0: no, 1: x, 2: y */


typedef struct
{
   int n;          /* no of variables                                   */
   double y, dy;   /* optimal objective function value and estim. error */
   double *x, *dx; /* optimal point and estim. error                    */
   int term, itno; /* term = VSSAend, itno = no of iterations           */
} MIN_SOLUTION;
/* define unconstrained solution */


ExC_ MIN_SOLUTION* CreateSolution( MIN_PROBLEM *minprob );
/* create solution */


ExC_ void DeleteSolution( MIN_SOLUTION **solut );
/* delete solution and free memory */


/*
*  PART II: Extensions for Constrained Minimization
*/


typedef struct
{
   int n;                                   /* no of variables           */
   double (*func)( double *x, void *data ); /* objective function (min)  */
   void *func_data;                         /* objective function param  */

   int m;                                   /* no of inequ function cons */
   double (**ief)( double *x, void *data ); /* vector of inequ fun <= 0  */
   void **ief_data;                         /* inequ constr param        */

   int k;                                   /* no of equ function constr */
   double (**eqf)( double *x, void *data ); /* vector of equ funcs =  0  */
   void **eqf_data;                         /* equ constr param          */

   double *x_min;                           /* x_min[0...n-1] min values */
   double *x_max;                           /* x_max[0...n-1] max values */

   int maxitno_vssa;                        /* max VSSA iteration no     */
   int maxitno_penalty;                     /* max penalty iteration no  */
   double eps;                              /* error of solution > 0     */

   double alpha;                            /* reflection coefficent > 0 */
   double beta;                             /* 0 < contraction coeff < 1 */
   double gamma;                            /* 1 < expansion coefficent  */
   double r0;                               /* initial scaling para > 0  */
   double rf;                               /* upscalung factor     > 1  */
} MIN_PROBLEM_C;
/* define constrained minimization problem */


ExC_ SIMPLEX* MakeFirstSimplexC( MIN_PROBLEM_C *minprob, double *r );
/* generate first simplex for iteration */


ExC_ int VSSAsolveC( MIN_PROBLEM_C *minprob, SIMPLEX *simp, double *r );
/* VSSA for a scaling vector r,  return termination code          */
/* -1: simp contains solution, everything else: return of VSSAend */


ExC_ int updateSRU( MIN_PROBLEM_C *minprob, SIMPLEX *simp,
                    double *r, double *u, double *uerr, double *ulast );
/* updating simp, r, uerr, and ulast, return 0 for temination, */
/* or no. of violated constraints                              */


typedef struct
{
   int n;              /* no of variables                                   */
   double y, dy;       /* optimal objective function value and estim. error */
   double *x, *dx;     /* optimal point and estim. error                    */
   int term, itnop;    /* term  = 0: no, 1: regular, 2: irregular           */
                       /* itnop = no of penalty iterations                  */
   int m;              /* no of inequation function constraints             */
   double *dief;       /* values of inequation functions                    */
   double *ui, *uierr; /* estim. Lagrange multipliers and errors            */
   int k;              /* no of equation function constraints               */
   double *deqf;       /* values of equation functions                      */
   double *ue, *ueerr; /* estim. Lagrange multipliers and errors            */
} MIN_SOLUTION_C;
/* define constrained solution */


ExC_ MIN_SOLUTION_C* CreateSolutionC( MIN_PROBLEM_C *minprob );
/* create solution */


ExC_ void DeleteSolutionC( MIN_SOLUTION_C **solut );
/* delete solution and frees memory */


#endif
