/*
*  Variable-Size Simplex Algorithm
*  version 19.5.2002
*  (c) 2002 Ingo D. Rullhusen <d01c@uni-bremen.de>
* 
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
* 
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
* 
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
*/


#include <stdlib.h>
#include <math.h>
#include "libvssa.h"


static double squ(double x){ return(x*x); }


void LibVSSAversion(int *major, int *minor, int *sub)
{ *major=LIBVSSA_MAJOR; *minor=LIBVSSA_MINOR; *sub=LIBVSSA_SUB; }


SIMPLEX* MakeFirstSimplex( MIN_PROBLEM *minprob )
{
   int i, n, p;
   double min, max;
   SIMPLEX *simp;
   
   simp = malloc( sizeof(SIMPLEX) );

   simp->n = minprob->n;

   simp->P = calloc( simp->n+1, sizeof(double *) );
   for ( i=0; i<simp->n+1; i++ ) simp->P[i] =  calloc( simp->n, sizeof(double) );

   for ( p=0; p<simp->n+1; p++ )
   {
      for ( n=0; n<simp->n; n++ )
      {
         if ( p == n+1 )
            simp->P[p][n] = simp->n * ( minprob->x_max[n] - minprob->x_min[n] )
                            + minprob->x_min[n];
         else
            simp->P[p][n] = minprob->x_min[n];
      }
   }

   simp->y = calloc( simp->n+1, sizeof(double) );
   min = max = simp->y[0] = minprob->func( simp->P[0], minprob->func_data );
   simp->yl = simp->yh = 0;
   for ( p=1; p<simp->n+1; p++ )
   {
      simp->y[p] = minprob->func( simp->P[p], minprob->func_data );
      if ( simp->y[p] > max )
      {
         max = simp->y[p];
         simp->yh = p;
      }
      else if ( simp->y[p] < min )
      {
         min = simp->y[p];
         simp->yl = p;
      }
   }

   simp->Ps  = calloc( simp->n, sizeof(double) );
   simp->Pss = calloc( simp->n, sizeof(double) );
   simp->Pq  = calloc( simp->n, sizeof(double) );

   return(simp);
}


void DeleteSimplex( SIMPLEX **simp )
{
   int i;

   if ( *simp == NULL ) return;

   for ( i=0; i<(*simp)->n+1; i++ ) if ( (*simp)->P[i] != NULL ) free((*simp)->P[i]);
   if ( (*simp)->P != NULL ) free((*simp)->P);
   
   if ( (*simp)->y != NULL ) free((*simp)->y);

   if ( (*simp)->Ps != NULL ) free((*simp)->Ps);
   if ( (*simp)->Pss != NULL ) free((*simp)->Pss);
   if ( (*simp)->Pq != NULL ) free((*simp)->Pq);

   free(*simp);
   *simp = NULL;
}


void VSSAstep( MIN_PROBLEM *minprob, SIMPLEX *simp )
{
   int i, n;
   double *P_t, min, max;

   for ( n=0; n<simp->n; n++ ) simp->Pq[n] = 0.0;
   for ( i=0; i<simp->n+1; i++ )
   {
      if ( i != simp->yh )
         for ( n=0; n<simp->n; n++ ) simp->Pq[n] += simp->P[i][n] / simp->n;
   }
   
   for ( n=0; n<simp->n; n++ ) simp->Ps[n] = (1.0+minprob->alpha) * simp->Pq[n]
                                             - minprob->alpha * simp->P[simp->yh][n];

   simp->ys = minprob->func( simp->Ps, minprob->func_data );

   /* 1 */
   if ( simp->ys < simp->y[simp->yl] )
   {
      for ( n=0; n<simp->n; n++ ) simp->Pss[n] = (1.0-minprob->gamma) * simp->Pq[n]
                                                 + minprob->gamma * simp->Ps[n];

      simp->yss = minprob->func( simp->Pss, minprob->func_data );

      /* 2 */
      if ( simp->yss < simp->y[simp->yl] )
      {
         P_t = simp->P[simp->yh];
         simp->P[simp->yh] = simp->Pss;
         simp->Pss = P_t;
         simp->y[simp->yh] = simp->yss;
      }
      else
      {
         P_t = simp->P[simp->yh];
         simp->P[simp->yh] = simp->Ps;
         simp->Ps = P_t;
         simp->y[simp->yh] = simp->ys;
      }

   }
   else
   {

      /* 3 */
      for ( i=0; i<simp->n; i++ ) if ( simp->ys <= simp->y[i] && i != simp->yh ) break;
      if ( i >= simp->n )
      {
      
         /* 4 */
         if ( simp->ys > simp->y[simp->yh] )
         {
            /* empty */
         }
         else
         {
            P_t = simp->P[simp->yh];
            simp->P[simp->yh] = simp->Ps;
            simp->Ps = P_t;
            simp->y[simp->yh] = simp->ys;
         }

         for ( n=0; n<simp->n; n++ ) simp->Pss[n] = (1.0-minprob->beta) * simp->Pq[n]
                                                    + minprob->beta * simp->P[simp->yh][n];

         simp->yss = minprob->func( simp->Pss, minprob->func_data );

         /* 5 */
         if ( simp->yss > simp->y[simp->yh] )
         {
            for ( i=0; i<simp->n+1; i++ )
            {
               if ( i == simp->yl ) continue;

               for ( n=0; n<simp->n; n++ )
               {
                  simp->P[i][n] = ( simp->P[i][n] + simp->P[simp->yl][n] ) * 0.5;
               }

               simp->y[i] = minprob->func( simp->P[i], minprob->func_data );
            }
         }
         else
         {
            P_t = simp->P[simp->yh];
            simp->P[simp->yh] = simp->Pss;
            simp->Pss = P_t;
            simp->y[simp->yh] = simp->yss;
         }

      }
      else
      {
         P_t = simp->P[simp->yh];
         simp->P[simp->yh] = simp->Ps; 
         simp->Ps = P_t;
         simp->y[simp->yh] = simp->ys;
      }

   }

   /* update */
   min = max = simp->y[0];
   simp->yl = simp->yh = 0;
   for ( i=1; i<simp->n+1; i++ )
   {
      if ( simp->y[i] < min )
      {
         min = simp->y[i];
         simp->yl = i;
      }
      else if ( simp->y[i] > max )
      {
         max = simp->y[i];
         simp->yh = i;
      }
   }

}


int VSSAend( double epsilon, SIMPLEX *simp )
{
   int i, j;
   double ymid, sq, lq;

/* x criterion */
   for ( j=0; j<simp->n; j++ )
   {
      simp->Pq[j] = 0.0;
      for ( i=0; i<simp->n+1; i++ )
         simp->Pq[j] += simp->P[i][j] / ( simp->n + 1 );
   }

   sq = 0.0;
   for ( i=0; i<simp->n+1; i++ )
   {
      lq = 0.0;
      for ( j=0; j<simp->n; j++ ) lq += squ( simp->P[i][j] - simp->Pq[j] );
      sq += lq / simp->n;
   }

   if ( sq < squ( epsilon ) ) return(1);

/* y criterion */
   ymid = 0.0;
   for ( i=0; i<simp->n+1; i++ )
   {
      ymid += simp->y[i] / ( simp->n + 1 );
   }

   sq = 0.0;
   for ( i=0; i<simp->n+1; i++ )
   {
      sq += squ( simp->y[i] - ymid ) / simp->n;
   }

   if ( sq < squ( epsilon ) ) return(2);

   return(0);
}


MIN_SOLUTION* CreateSolution( MIN_PROBLEM *minprob )
{
   int i, j;
   SIMPLEX *simp;
   MIN_SOLUTION *ms;

   simp = MakeFirstSimplex(minprob);

   ms = malloc( sizeof(MIN_SOLUTION) );
   ms->n = minprob->n;
   ms->term = 0;

   for ( i=0; i<minprob->maxitno; i++ )
   {
      if ( ( ms->term = VSSAend(minprob->eps,simp) ) ) break;

      VSSAstep( minprob, simp );

   }
   ms->itno = i;

   ms->y = simp->y[simp->yl];
   ms->dy = simp->y[simp->yh] - simp->y[simp->yl];

   ms->x = calloc( simp->n, sizeof(double) );
   ms->dx = calloc( simp->n, sizeof(double) );
   for ( i=0; i<simp->n; i++ )
   {
      ms->x[i] = simp->P[simp->yl][i];
      ms->dx[i] = 0.0;
      for ( j=0; j<simp->n+1; j++ )
         if ( ms->dx[i] < fabs( ms->x[i] - simp->P[j][i] ) )
            ms->dx[i] = fabs( ms->x[i] - simp->P[j][i] );
   }

   DeleteSimplex(&simp);

   return(ms);
}


void DeleteSolution( MIN_SOLUTION **solut )
{
   if ( *solut == NULL ) return;
   
   if ( (*solut)->x != NULL ) free((*solut)->x);
   if ( (*solut)->dx != NULL ) free((*solut)->dx);

   free(*solut);
   *solut = NULL;
}
