/*
*  Variable-Size Simplex Algorithm
*  version 15.4.2002
*  (c) 2002 Ingo D. Rullhusen <d01c@uni-bremen.de>
* 
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
* 
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
* 
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
*/


#include <stdlib.h>
#include <math.h>
#include "libvssa.h"


static double squ(double x){ return(x*x); }


typedef struct
{
   int m;                                   /* no of inequ constraints   */
   double (**ief)( double *x, void *data ); /* vector of inequ f <= 0    */  
   void **ief_data;                         /* inequ constr param        */
   int k;                                   /* no of equ constraints     */
   double (**eqf)( double *x, void *data ); /* vector of equ f   =  0    */   
   void **eqf_data;                         /* equ constr param          */
   double *r;                               /* scaling factors           */
} CONSTRAINT;


static double Penalty( double *x, CONSTRAINT *c )
{
   int i;
   double p, g;

   p = 0.0;
   for ( i=0; i<c->m; i++ ) p += c->r[ i ]
                                 * ( ( g=c->ief[i]( x, c->ief_data[i] )
                                     ) > 0.0 ? squ(g) : 0.0 );
   for ( i=0; i<c->k; i++ ) p += c->r[ c->m + i ]
                                 * squ( c->eqf[i]( x, c->eqf_data[i] ) );

   return(p);
}


typedef struct
{
   double (*func)( double *x, void *data ); /* objective function (min)  */
   void *func_data;                         /* objective function param  */
   CONSTRAINT c;                            /* constraints               */
} MinFuncData;


static double MinFunc( double *x, void *data )
{
   MinFuncData *mfd = (MinFuncData*)data;

   return( mfd->func( x, mfd->func_data ) +  Penalty( x, &mfd->c ) );
}


SIMPLEX* MakeFirstSimplexC( MIN_PROBLEM_C *minprob_c, double *r )
{
   MIN_PROBLEM minprob;
   MinFuncData MinFuncData;

   minprob.n         = minprob_c->n;
   minprob.func      = MinFunc;
   minprob.func_data = (void*)&MinFuncData;

   MinFuncData.func       = minprob_c->func;
   MinFuncData.func_data  = minprob_c->func_data;
   MinFuncData.c.m        = minprob_c->m;
   MinFuncData.c.ief      = minprob_c->ief;
   MinFuncData.c.ief_data = minprob_c->ief_data;
   MinFuncData.c.k        = minprob_c->k;
   MinFuncData.c.eqf      = minprob_c->eqf;
   MinFuncData.c.eqf_data = minprob_c->eqf_data;
   MinFuncData.c.r        = r;

   minprob.x_min = minprob_c->x_min;
   minprob.x_max = minprob_c->x_max;

   minprob.maxitno = minprob_c->maxitno_vssa;
   minprob.eps     = minprob_c->eps;
   minprob.alpha   = minprob_c->alpha;
   minprob.beta    = minprob_c->beta;
   minprob.gamma   = minprob_c->gamma;

   return( MakeFirstSimplex( &minprob ) );
}


int VSSAsolveC( MIN_PROBLEM_C *minprob_c, SIMPLEX *simp, double *r )
{
   int i, term = 0;
   MIN_PROBLEM minprob;
   MinFuncData MinFuncData;

   minprob.n         = minprob_c->n;
   minprob.func      = MinFunc;
   minprob.func_data = (void*)&MinFuncData;

   MinFuncData.func       = minprob_c->func;
   MinFuncData.func_data  = minprob_c->func_data;
   MinFuncData.c.m        = minprob_c->m;
   MinFuncData.c.ief      = minprob_c->ief;
   MinFuncData.c.ief_data = minprob_c->ief_data;
   MinFuncData.c.k        = minprob_c->k;
   MinFuncData.c.eqf      = minprob_c->eqf;
   MinFuncData.c.eqf_data = minprob_c->eqf_data;
   MinFuncData.c.r        = r;

   minprob.x_min = minprob_c->x_min;
   minprob.x_max = minprob_c->x_max;

   minprob.maxitno = minprob_c->maxitno_vssa;
   minprob.eps     = minprob_c->eps;
   minprob.alpha   = minprob_c->alpha;
   minprob.beta    = minprob_c->beta;
   minprob.gamma   = minprob_c->gamma;

   for ( i=0; i<minprob.maxitno; i++ )
   {
      if ( ( term=VSSAend(minprob.eps,simp) ) )
      {
         if ( i == 0 ) term = -1;
         break;
      }

      VSSAstep( &minprob, simp );

   }

   return(term);
}


int updateSRU( MIN_PROBLEM_C *minprob_c, SIMPLEX *simp,
               double *r, double *u, double *uerr, double *ulast )
{
   int i, j, l, term = 0;
   double min, max, gh;
   MinFuncData MinFuncData;

   for ( i=0; i<minprob_c->m; i++ )
   {
      if ( ( gh=minprob_c->ief[i]( simp->P[simp->yl], minprob_c->ief_data[i] )
           ) > fabs(minprob_c->eps) )
      {
         gh = r[i] * (2.0*gh);
         if ( fabs( ( gh - ulast[i] ) / gh ) < fabs(uerr[i]) )
         {
            uerr[i] = fabs( ( gh - u[i] ) / gh );
            u[i] = gh;
         }
         ulast[i] = gh;
         term++;
         r[i] *= minprob_c->rf;
      }
   }

   for ( i=0; i<minprob_c->k; i++ )
   {
      if ( fabs( ( gh=minprob_c->eqf[i]( simp->P[simp->yl],
                                         minprob_c->eqf_data[i] ) )
               ) > fabs(minprob_c->eps) )
      {
         gh = r[minprob_c->m+i] * (2.0*gh);
         if ( fabs( ( gh - ulast[minprob_c->m+i] ) / gh )
              < fabs(uerr[minprob_c->m+i]) )
         {
            uerr[minprob_c->m+i] = fabs( ( gh - u[minprob_c->m+i] ) / gh );
            u[minprob_c->m+i] = gh;
         }
         ulast[minprob_c->m+i] = gh;
         term++;
         r[minprob_c->m+i] *= minprob_c->rf;
      }
   }

   if ( term )
   {
      MinFuncData.func       = minprob_c->func;
      MinFuncData.func_data  = minprob_c->func_data;
      MinFuncData.c.m        = minprob_c->m;
      MinFuncData.c.ief      = minprob_c->ief;
      MinFuncData.c.ief_data = minprob_c->ief_data;
      MinFuncData.c.k        = minprob_c->k;
      MinFuncData.c.eqf      = minprob_c->eqf;
      MinFuncData.c.eqf_data = minprob_c->eqf_data;
      MinFuncData.c.r        = r;

      l = simp->yl;
      min = max = simp->y[0] = MinFunc( simp->P[0], &MinFuncData );
      simp->yl = simp->yh = 0;
      for ( i=1; i<simp->n+1; i++ )
      {
         for ( j=0; j<simp->n; j++ )
            simp->P[i][j] = 2.0*simp->P[i][j] - simp->P[l][j];

         simp->y[i] = MinFunc( simp->P[i], &MinFuncData );
         if ( simp->y[i] > max )
         {
            max = simp->y[i];
            simp->yh = i;
         }
         else if ( simp->y[i] < min )
         {
            min = simp->y[i];
            simp->yl = i;
         }
      }

   }

   return(term);
}


MIN_SOLUTION_C* CreateSolutionC( MIN_PROBLEM_C *minprob )
{
   int i, j, term;
   double *r, *u, *uerr, *ulast;
   SIMPLEX *simp;
   MIN_SOLUTION_C *ms;
   
   r = calloc( minprob->m+minprob->k, sizeof(double) );
   u = calloc( minprob->m+minprob->k, sizeof(double) );
   uerr = calloc( minprob->m+minprob->k, sizeof(double) );
   ulast = calloc( minprob->m+minprob->k, sizeof(double) );
   for ( i=0; i<minprob->m+minprob->k; i++ )
   {
      r[i] = minprob->r0;
      u[i] = 0.0;
      ulast[i] = 0.0;
   }

   simp = MakeFirstSimplexC(minprob,r);

   for ( i=0; i<minprob->m; i++ ) uerr[i] = - 2.0;
   for ( i=0; i<minprob->k; i++ ) uerr[minprob->m+i] = - 2.0;

   ms = malloc( sizeof(MIN_SOLUTION_C) );
   ms->n = minprob->n;
   ms->m = minprob->m;
   ms->k = minprob->k;

   ms->term = 0;
   for ( i=0; i<minprob->maxitno_penalty; i++ )
   {
      if ( ( term=VSSAsolveC( minprob, simp, r ) ) < 1 )
      {
         if ( term == -1 ) ms->term = 2;
         break;
      }
      if ( updateSRU( minprob, simp, r, u, uerr, ulast ) == 0 )
      {
         ms->term = 1;
         break;
      }
   }
   ms->itnop = i;

   ms->y = minprob->func( simp->P[simp->yl], minprob->func_data );
   ms->dy = fabs( minprob->func( simp->P[simp->yh], minprob->func_data )
                  - ms->y );

   ms->x = calloc( simp->n, sizeof(double) );
   ms->dx = calloc( simp->n, sizeof(double) );
   for ( i=0; i<simp->n; i++ )
   {
      ms->x[i] = simp->P[simp->yl][i];
      ms->dx[i] = 0.0;
      for ( j=0; j<simp->n+1; j++ )
         if ( ms->dx[i] < fabs( ms->x[i] - simp->P[j][i] ) )
            ms->dx[i] = fabs( ms->x[i] - simp->P[j][i] );
   }

   DeleteSimplex(&simp);

   ms->dief = calloc( ms->m, sizeof(double) );
   ms->ui = calloc( ms->m, sizeof(double) );
   ms->uierr = calloc( ms->m, sizeof(double) );
   for ( i=0; i<ms->m; i++ )
   {
      ms->dief[i] = minprob->ief[i]( ms->x, minprob->ief_data[i] );
      ms->ui[i] = u[i];
      if ( uerr[i] < 0.0 )
      {
         ms->ui[i] = 0.0;
         ms->uierr[i] = 0.0;
      }
      else
      {
         ms->uierr[i] = fabs( uerr[i] * u[i] );
      }
   }

   ms->deqf = calloc( ms->k, sizeof(double) );
   ms->ue = calloc( ms->k, sizeof(double) );
   ms->ueerr = calloc( ms->k, sizeof(double) );
   for ( i=0; i<ms->k; i++ )
   {
      ms->deqf[i] = minprob->eqf[i]( ms->x, minprob->eqf_data[i] );
      ms->ue[i] = u[ms->m+i];
      if ( uerr[ms->m+i] < 0.0 )
      {
         ms->ue[i] = 0.0;
         ms->ueerr[i] = 0.0;
      }
      else
      {
         ms->ueerr[i] = fabs( uerr[ms->m+i] * u[ms->m+i] );
      }
   }

   free(ulast);
   free(uerr);
   free(u);
   free(r);

   return(ms);
}


void DeleteSolutionC( MIN_SOLUTION_C **solut )
{
   if ( *solut == NULL ) return;
   
   if ( (*solut)->x != NULL ) free((*solut)->x);
   if ( (*solut)->dx != NULL ) free((*solut)->dx);
   
   if ( (*solut)->ui != NULL ) free((*solut)->ui);
   if ( (*solut)->ue != NULL ) free((*solut)->ue);

   if ( (*solut)->uierr != NULL ) free((*solut)->uierr);
   if ( (*solut)->ueerr != NULL ) free((*solut)->ueerr);

   if ( (*solut)->dief != NULL ) free((*solut)->dief);
   if ( (*solut)->deqf != NULL ) free((*solut)->deqf);

   free(*solut);
   *solut = NULL;
}
